﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TwelvePx.Core;
using TwelvePx.Graphics;
using TwelvePx.Interfaces;
using SFML.Graphics;
using SFML.Window;
using TwelvePx.Elements;
using TwelvePx.Tools;

namespace TwelvePx.Elements
{
    public class Map : Drawable
    {
        GraphicsBuffer buffer;

        public uint Columns { get; set; }
        public uint Rows { get; set; }

        public uint StartRow { get; set; }
        public uint StartColumn { get; set; }

        public BlockType[] Blocks { get; private set; }

        public Map(uint ScreenWidth, uint ScreenHeight, uint MapColumns, uint MapRows)
        {
            buffer = new GraphicsBuffer(ScreenWidth, ScreenHeight);

            Blocks = new BlockType[MapColumns * MapRows];

            Columns = MapColumns;
            Rows = MapRows;

            FillWithVoid();
        }

        public Map(uint ScreenWidth, uint ScreenHeight, string Data)
        {
            buffer = new GraphicsBuffer(ScreenWidth, ScreenHeight);

            Data = Data.Replace("\n\r", "\n").Replace("\r\n", "\n").Replace("\r", "\n");
            string[] lines = Data.Split('\n');

            if (lines.Length == 0)
                return;
            Rows = (uint)lines.Length;
            Columns = (uint)lines[0].Length;

            Blocks = new BlockType[Columns * Rows];
            int x = 0;
            int y = 0;

            foreach (string line in lines)
            {
                foreach (char chr in line)
                {
                    if (char.IsNumber(chr))
                    {
                        Blocks[(y * Columns) + x] = (BlockType)Enum.Parse(typeof(BlockType), chr.ToString());
                        x++;
                        continue;
                    }
                    else
                    {
                        switch (chr)
                        {
                            case 'X':
                                Blocks[(y * Columns) + x] = BlockType.VoidBlock;
                                StartRow = (uint)y;
                                StartColumn = (uint)x;
                                x++;
                                continue;
                            case 'T':
                                Blocks[(y * Columns) + x] = BlockType.FinishBlock;
                                x++;
                                continue;
                            case ' ':
                                Blocks[(y * Columns) + x] = BlockType.VoidBlock;
                                x++;
                                continue;
                            case '#':
                                Blocks[(y * Columns) + x] = BlockType.SolidBlock;
                                x++;
                                continue;
                        }
                    }
                }
                y++;
                x = 0;
            }
        }

        public void RenderRect(uint Column, uint Row)
        {
            if (Column > Columns - 1)
                throw new ArgumentOutOfRangeException("Column");
            if (Row > Rows - 1)
                throw new ArgumentOutOfRangeException("Row");

            buffer.Fill(Color.Transparent);

            for (uint x = 0; x < buffer.Width; x++)
            {
                for (uint y = 0; y < buffer.Height; y++)
                {
                    if (((y + Row) * Columns) + x + Column >= Blocks.Length)
                        continue;
                    buffer.Set(x, y, Block.BlockLookup[Blocks[((y + Row) * Columns) + x + Column]]);
                }
            }
        }

        public void RenderPlayerRect(Player Player)
        {
            //if (Player.Row > 1 && Player.Column > 1)
           // {
                RenderRect(Player.Column - 1, Player.Row - 1);
            //}
            //else
            //{
                
            //}
        }

        public void FillWithVoid()
        {
            for (uint x = 0; x < Columns; x++)
            {
                for (uint y = 0; y < Rows; y++)
                {
                    Blocks[(y * Columns) + x] = BlockType.VoidBlock;
                }
            }
        }

        public void DrawBorder()
        {
            DrawSolidHLine(0);
            DrawSolidHLine(-1);
            DrawSolidVLine(0);
            DrawSolidVLine(-1);
        }

        public void DrawSolidHLine(int Row, uint StartColumn = 0, int EndColumn = -1)
        {
            Row = ((Row < 0) ? (int)Rows - 1 : Row);
            EndColumn = ((EndColumn < 0) ? (int)Columns - 1 : EndColumn);

            for (uint x = StartColumn; x < EndColumn; x++)
                Set(x, (uint)Row, BlockType.SolidBlock);
        }

        public void DrawSolidVLine(int Column, uint StartRow = 0, int EndRow = -1)
        {
            Column = ((Column < 0) ? (int)Columns - 1 : Column);
            EndRow = ((EndRow < 0) ? (int)Rows - 1 : EndRow);

            for (uint y = StartRow; y < EndRow; y++)
                Set((uint)Column, y, BlockType.SolidBlock);
        }

        public void SelectRect(Vector2u Position)
        {
            RenderRect(Position.X, Position.Y);
        }

        public void Set(uint Column, uint Row, BlockType Block)
        {
            if (Column > Columns)
                throw new ArgumentOutOfRangeException("Column");
            if (Row > Rows)
                throw new ArgumentOutOfRangeException("Row");

            Blocks[(Row * Columns) + Column] = Block;
        }

        public void Set(Vector2u Position, BlockType Block)
        {
            Set(Position.X, Position.Y, Block);
        }

        public BlockType Get(uint Column, uint Row)
        {
            if (Column > Columns)
                throw new ArgumentOutOfRangeException("Column");
            if (Row > Rows)
                throw new ArgumentOutOfRangeException("Row");

            return Blocks[(Row * Columns) + Column];
        }

        public BlockType Get(Vector2u Position)
        {
            return Get(Position.X, Position.Y);
        }

        public void Draw(RenderTarget target, RenderStates states)
        {
            target.Draw(buffer, states);
        }
    }
}
