﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SFML.Graphics;
using SFML.Window;
using TwelvePx.Core;
using TwelvePx.Graphics;
using TwelvePx.Interfaces;
using TwelvePx.Tools;

namespace TwelvePx.Elements
{
    public class Player : Drawable
    {
        RectangleShape rect;

        public GameContext Context {get; private set;}

        public Player(GameContext Context)
        {
            this.Context = Context;
            rect = new RectangleShape(new Vector2f(1, 1));
            rect.FillColor = Color.Green;
            rect.Position = new Vector2f(1, 1);
        }

        public void Draw(RenderTarget target, RenderStates states)
        {
            target.Draw(rect, states);
        }

        public void SetPosition(uint Column, uint Row)
        {
            this.Column = Column;
            this.Row = Row;
        }

        public void RelativeMove(int Columns, int Rows)
        {
            Column = (uint)(((int)Column) + Columns);
            Row = (uint)(((int)Row) + Rows);
            Block.Touch(this, Context.CurrentMap, Context.CurrentMap.Get(Column, Row));
        }

        public uint OriginRow
        {
            get
            {
                return ((uint)rect.Position.Y);
            }

            set
            {
                rect.Position = new Vector2f(rect.Position.X, (float)value);
            }
        }

        public uint OriginColumn
        {
            get
            {
                return ((uint)rect.Position.X);
            }

            set
            {
                rect.Position = new Vector2f((float)value, rect.Position.Y);
            }
        }

        public uint Row { get; set; }

        public uint Column { get; set; }

        public bool CanMoveUp
        {
            get
            {
                return ((((int)Row) - 1 >= 0) ? Block.CanTouch(Context.CurrentMap.Get(Column, Row - 1)) : false);
            }
        }

        public bool CanMoveDown
        {
            get
            {
                return ((((int)Row) + 1 < Context.CurrentMap.Rows) ? Block.CanTouch(Context.CurrentMap.Get(Column, Row + 1)) : false);
            }
        }

        public bool CanMoveRight
        {
            get
            {
                return ((((int)Column) + 1 < Context.CurrentMap.Columns) ? Block.CanTouch(Context.CurrentMap.Get(Column + 1, Row)) : false);
            }
        }

        public bool CanMoveLeft
        {
            get
            {
                return ((((int)Column) - 1 >= 0) ? Block.CanTouch(Context.CurrentMap.Get(Column - 1, Row)) : false);
            }
        }
    }
}
