﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SFML.Graphics;
using SFML.Window;
using TwelvePx.Interfaces;
using TwelvePx.Core;

namespace TwelvePx.Graphics
{
    public class GraphicsBuffer : Drawable
    {
        private RectangleShape[] rects;

        public uint Width { get; set; }
        public uint Height { get; set; }

        public GraphicsBuffer(uint Width, uint Height)
        {
            rects = new RectangleShape[Width * Height];

            this.Width = Width;
            this.Height = Height;

            for (uint y = 0; y < Height; y++)
            {
                for (uint x = 0; x < Width; x++)
                {
                    var rect = new RectangleShape(new Vector2f(1, 1));
                    rect.Position = new Vector2f(x, y);

                    rects[(y * Width) + x] = rect;
                }
            }

            Fill(Color.Transparent);
        }

        public void Set(uint X, uint Y, Color Color)
        {
            if (X > Width)
                throw new ArgumentOutOfRangeException("X");
            if (Y > Height)
                throw new ArgumentOutOfRangeException("Y");

            rects[(int)((Y * Width) + X)].FillColor = Color;
        }

        public void Set(Vector2u Position, Color Color)
        {
            Set(Position.X, Position.Y, Color);
        }

        public Color Get(uint X, uint Y)
        {
            if (X > Width)
                throw new ArgumentOutOfRangeException("X");
            if (Y > Height)
                throw new ArgumentOutOfRangeException("Y");

            return rects[(int)((Y * Width) + X)].FillColor;
        }

        public Color Get(Vector2u Position)
        {
            return Get(Position.X, Position.Y);
        }

        public void Draw(RenderTarget target, RenderStates states)
        {
            foreach (RectangleShape rect in rects)
                target.Draw(rect, states);
        }

        public void Fill(Color Color)
        {
            foreach (RectangleShape rect in rects)
                rect.FillColor = Color;
        }

        public void Destroy()
        {
            if (rects.Length == 0)
                return;

            for (int i = rects.Length - 1; i > 0; i++)
                rects[i].Dispose();
        }
    }
}
